import MaxFlow.MaxFlow;
import org.graphstream.graph.Edge;
import org.graphstream.graph.Graph;
import org.graphstream.graph.implementations.Graphs;
import org.graphstream.graph.implementations.SingleGraph;
import org.graphstream.stream.GraphParseException;

import java.io.IOException;
import java.util.ArrayList;

public class Autoroute
{

	public static void main(String[] args) throws IOException, GraphParseException
	{
		Graph g = new SingleGraph("Autoroute");
		g.read("data/reseauRoutier.dgs");

		g.setAttribute("ui.quality");
		g.setAttribute(("ui.antialias"));
		g.setAttribute("ui.stylesheet", "url('data/style.css')");
		g.nodes().forEach(n -> n.setAttribute("label", n.getId()));

		System.setProperty("org.graphstream.ui", "swing");

		Autoroute.flotOptimal(g);
		MaxFlow mf = Autoroute.maxFlot(g);


		System.out.println(mf.getFlow());
		g.edges().forEach(e -> {
			double flow = mf.getFlow(e);
			double cap = mf.getCapacity(e);
			e.setAttribute("ui.label", "" + (int)flow+"/" + e.getAttribute("cap"));
			if (cap == flow) e.setAttribute("ui.style", "fill-color: red;");
		});
		g.display(false);

	}

	/**
	 * Méthode qui lance l'algorithme de flowmax
	 * @param g
	 * @return
	 */
	private static MaxFlow maxFlot(Graph g)
	{
		MaxFlow mf = new MaxFlow();
		mf.setCapacityAttribute("cap");
		mf.init(g);
		mf.setSource(g.getNode("A"));
		mf.setSink(g.getNode("I"));
		mf.compute();
		return mf;
	}

	/**
	 * Méthode qui donne les flots successifs
	 * @param g
	 */
	private static void flotOptimal(Graph g)
	{
		int arcCap;
		double flotMax;
		Edge troncon;
		MaxFlow mf;
		Graph g2;
		String[] arcChoisi = {"AC", "CD", "DH", "HI"};
		ArrayList<Edge> arcParcourir = Autoroute.getTroncon(g, arcChoisi);

		do
		{
			flotMax = 0.0;
			troncon = null;

			for (Edge e:arcParcourir)
			{
				arcCap = (int) e.getAttribute("cap");

				// On ajoute à la capacité maximale du troncon de 2500
				e.setAttribute("cap", 2500 + arcCap);

				// Lancement de l'algorithme de maxflow
				mf = Autoroute.maxFlot(g);

				//Permet de stocker le troncon soit optimal ainsi que le flotmax
				if(flotMax < mf.getFlow())
				{
					troncon = e;
					flotMax = mf.getFlow();
				}
				e.setAttribute("cap", arcCap);
			}

			//On met à jour la capacité maximale du troncon optimal
			troncon.setAttribute("cap", (int) troncon.getAttribute("cap")+2500);

			System.out.println("Arc augmenté: [" + troncon.getSourceNode()+"->"+ troncon.getTargetNode()+ "] | flot max : " + flotMax);

			arcParcourir.remove(troncon);

			// Affiche le graphe
			g2 = Graphs.clone(g);
			Autoroute.afficherGraphe(g2);


		}while(!arcParcourir.isEmpty());

	}

	/**
	 * Méthodee qui permet d'afficher le graphe en paramètre
	 * @param g2
	 */
	private static void afficherGraphe(Graph g2)
	{
		MaxFlow mf2 = Autoroute.maxFlot(g2);

		g2.edges().forEach(e -> {
			double flow = mf2.getFlow(e);
			double cap = mf2.getCapacity(e);
			e.setAttribute("ui.label", "" + (int)flow+"/" + e.getAttribute("cap"));
			if (cap == flow) e.setAttribute("ui.style", "fill-color: red;");
		});
		g2.display(false);
	}

	/**
	 * Recupere les noeuds passé en paramètre
	 * @param g
	 * @param arcChoisi
	 * @return
	 */
	private static ArrayList<Edge> getTroncon(Graph g, String[] arcChoisi)
	{
		ArrayList<Edge> alEdge = new ArrayList<>();

		for (int i = 0; i < arcChoisi.length; i++)
			alEdge.add(g.getEdge(arcChoisi[i]));

		return alEdge;

	}

}
